/*********************************************************************
*                    SEGGER Microcontroller GmbH                     *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*            (c) 2014 - 2019 SEGGER Microcontroller GmbH             *
*                                                                    *
*           www.segger.com     Support: support@segger.com           *
*                                                                    *
**********************************************************************
*                                                                    *
* All rights reserved.                                               *
*                                                                    *
* Redistribution and use in source and binary forms, with or         *
* without modification, are permitted provided that the following    *
* conditions are met:                                                *
*                                                                    *
* - Redistributions of source code must retain the above copyright   *
*   notice, this list of conditions and the following disclaimer.    *
*                                                                    *
* - Neither the name of SEGGER Microcontroller GmbH                  *
*   nor the names of its contributors may be used to endorse or      *
*   promote products derived from this software without specific     *
*   prior written permission.                                        *
*                                                                    *
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND             *
* CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,        *
* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF           *
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE           *
* DISCLAIMED.                                                        *
* IN NO EVENT SHALL SEGGER Microcontroller GmbH BE LIABLE FOR        *
* ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR           *
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT  *
* OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;    *
* OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF      *
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT          *
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE  *
* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH   *
* DAMAGE.                                                            *
*                                                                    *
**********************************************************************

-------------------------- END-OF-HEADER -----------------------------

File    : main.c
Purpose : Generic application start

*/

#include <stdio.h>
#include <stdlib.h>

/*********************************************************************
*
*       _IllegalRead()
*
*  Function description
*   Trigger a bus fault or hard fault by reading from a reserved address.
*
*  Additional Information
*    Bus fault is immediately triggered on the read instruction.
*    Related NVIC registers on hard fault:
*      NVIC.HFSR = 0x40000000
*        FORCED = 1           - bus fault/memory management fault/usage fault escalated to hard fault
*      NVIC.BFSR = 0x00000082
*        PRECISERR = 1        - Precise data access violation
*        BFARVALID = 1        - BFAR is valid
*      NVIC.BFAR = 0x00100000 - The address read from
*/
static int _IllegalRead(void) {
  int r;
  volatile unsigned int* p;

  p = (unsigned int*)0x00100000;  // 0x00100000-0x07FFFFFF is reserved on STM32F4
  r = *p;

  return r;
}

/*********************************************************************
*
*      _FuncB
*
*  Function description
*   Simple call path, level 2
*/
static int _FuncB (int b) {
  return _IllegalRead();
}

/*********************************************************************
*
*      _FuncA
*
*  Function description
*   Simple call path, level 1
*/
static int _FuncA (int a) {
  return _FuncB(a) + a;
}

/*********************************************************************
*
*      _CallSupervisorIn0
*
*  Function description
*   Simple call path to supervisor call, level 0
*/
static void _CallSupervisorIn0(int Level) {

  if (Level == 2) {
    asm("svc #0");
  }
}

/*********************************************************************
*
*      _CallSupervisorIn1
*
*  Function description
*   Simple call path to supervisor call, level 1
*/
static void _CallSupervisorIn1(int Level) {
  _CallSupervisorIn0(Level + 1);
}

/*********************************************************************
*
*      _CallSupervisorIn2
*
*  Function description
*   Simple call path to supervisor call, level 2
*/
static void _CallSupervisorIn2(int Level) {

  _CallSupervisorIn1(Level + 1);
}

/*********************************************************************
*
*      SVC_Handler
*
*  Function description
*   software interrupt exception handler
*/
void SVC_Handler (void) {
  
  if (_FuncA(1)) {
    _FuncB(0);
  }
}

/*********************************************************************
*
*       main()
*
*  Function description
*   Application entry point.
*/
void main(void) {
  int i;
  //
  // set "Process stack size" in project settings to:
  //
  //   = 0: application runs on main stack (MSP),    exception frame generated on main stack
  //   > 0: application runs on process stack (PSP), exception frame generated on process stack
  //
  for (i = 0; i < 100; i++) {

    printf("Hello World %d!\n", i);
    
    if (i % 10 == 0) {
      _CallSupervisorIn2(0);
    }
  }
  do {
    i++;
  } while (1);
}

/*************************** End of file ****************************/
